package com.myoa.engineering.crawl.ppomppu.processor.controller;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.myoa.engineering.crawl.ppomppu.processor.dto.FeedParsedResult;
import com.myoa.engineering.crawl.ppomppu.processor.service.MessageSenderService;
import com.myoa.engineering.crawl.ppomppu.processor.service.PpomppuArticleService;
import com.myoa.engineering.crawl.ppomppu.processor.service.PpomppuFeedService;
import com.myoa.engineering.crawl.ppomppu.support.dto.APIResponse;
import com.myoa.engineering.crawl.ppomppu.support.dto.code.PpomppuBoardName;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

/**
 * CrawlAPIController
 *
 * @author Shin Woo-jin (woo-jin.shin@linecorp.com)
 * @since 2021-09-05
 */
@Slf4j
@RestController
@RequestMapping("/api/v1/crawl")
public class CrawlAPIController {

    private final PpomppuFeedService ppomppuRSSFeedService;
    private final PpomppuArticleService ppomppuArticleService;
    private final MessageSenderService messageSenderService;

    public CrawlAPIController(PpomppuFeedService ppomppuRSSFeedService,
                              PpomppuArticleService ppomppuArticleService,
                              MessageSenderService messageSenderService) {
        this.ppomppuRSSFeedService = ppomppuRSSFeedService;
        this.ppomppuArticleService = ppomppuArticleService;
        this.messageSenderService = messageSenderService;
    }

    @PostMapping("/boards/{boardName}")
    public Mono<APIResponse<FeedParsedResult>> crawlBoard(@PathVariable("boardName") PpomppuBoardName boardName) {
        log.info("got request... {}", boardName);
        FeedParsedResult result = FeedParsedResult.of(boardName);

        Mono<String> publishedMessages =
            ppomppuRSSFeedService.getArticles(boardName)
                                 .map(e -> ppomppuArticleService.filterOnlyNewArticles(boardName, e))
                                 .map(e -> ppomppuArticleService.save(boardName, e))
                                 .flatMap(messageSenderService::sendMessageToSlack);

        return publishedMessages.then(Mono.just(APIResponse.success(result.done())));
    }

}
